/*
 * Chromosome.java
 *
 * Created on Nedea, 2007, aprl 29, 18:12
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package spectKonfo;

/**
 * Trieda <CODE>Chromosome</CODE> obsahuje predpokladan spektr farbv, ktor s sasou vzorky. alej obsahuje metdy nutn pre evolciu v rmci genetickho algoritmu.
 * @author Kamil Paulny
 */
public class Chromosome implements Cloneable {
    /**
     * Kontanta pre UniversalSpectrum
     */
    public static final int UNIVERSAL = 0;
    /**
     * Kontanta pre GLSpectrum
     */
    public static final int GL = 1;
    /**
     * Kontanta pre GLMSpectrum tvoren dvoma rozdeleniami.
     */
    public static final int GLM2 = 2;
    /**
     * Kontanta pre GLMSpectrum tvoren troma rozdeleniami.
     */
    public static final int GLM3 = 3;
    /**
     * Kontanta pre UniformSpectrum
     */
    public static final int UNIFORM = 4;
    /**
     * Predvolen typ Spektra
     */
    public static final int TYPE_DEFAULT = GL;
    /**
     * Pravdepodobnos krenia
     */
    public static final double PCROSS = 0.5;
    private Spectrum[] spectrum; //pole spektier, ktor vytvraj spektrlnu maticu A
    private int m; //poet spektier
    private int n; //n x m s rozmery spektrlnej matice
    /**
     * Sila chromozmu.
     */
    double fitness;
    /**
     * Spektrlna matica.
     */
    double[][] A; //spektrlna matica
    /** Creates a new instance of Chromosome */
    
    private static int[] makeArray(int m, int x) {
        int[] type = new int[m];
        for (int i = 0; i < m; i++) {
            type[i] = x;
        }
        return type;
    }
    
    /**
     * Vytvor nov intanciu triedy <CODE>Chromosome</CODE>. Bude obsahova <CODE>m</CODE> spektier. Jedno bude typu UniformSpectrum a zvyn bud vytvoren pomocou daja v kontante TYPE_DEFAULT.
     * @param n
     * @param m Poet spektier.
     * @param minWaveLength
     * @param step
     */
    public Chromosome(int n, int m, int minWaveLength, int step) {
        this(n, makeArray(m, TYPE_DEFAULT), minWaveLength, step);
    }
    
    /**
     * Vytvor nov intanciu triedy <CODE>Chromosome</CODE>. Spektr bud vytvoren poda dajov v poli <CODE>type</CODE>.
     * @param n
     * @param type Mala by obsahova statick kontanty tejto triedy. Ak sa niektor hodnota nezhoduje so iadnou kontantou, je pouit kontanta TYPE_DEFAULT
     * @param minWaveLength
     * @param step
     */
    public Chromosome(int n, int[] type, double minWaveLength, double step) {
        m = type.length;
        this.n = n;
        spectrum = new Spectrum[m];
        for(int i = 0; i < m; i++) {
            if (type[i] == GLM2)
                spectrum[i] = new GLMSpectrum(n, minWaveLength, step, 2);
            else if (type[i] == GLM3)
                spectrum[i] = new GLMSpectrum(n, minWaveLength, step, 3);
            else if (type[i] == UNIVERSAL)
                spectrum[i] = new UniversalSpectrum(n);
            else if (type[i] == UNIFORM)
                spectrum[i] = new UniformSpectrum(n);
            else
                spectrum[i] = new GLSpectrum(n, minWaveLength, step);
        }
        A = new double[n][m];
    }
    
    /**
     * Nastane krenie medzi tmto chromozmom a chromozmom <CODE>c</CODE>. S pravdepodobnosou PCROSS si vymenia korepondujce spektr. Okrem toho zavol krenie aj pre korepondujce dvojice spektier.
     * @param c
     */
    public void cross(Chromosome c) {
        Spectrum s;
        for(int i = 0; i < m; i++)
            if(RandomUtils.nextDouble()<PCROSS) {
            s = this.spectrum[i];
            this.spectrum[i] = c.spectrum[i];
            c.spectrum[i] = s;
            }
        for(int i = 0; i < m; i++)
            this.spectrum[i].cross(c.spectrum[i]);
    }
    
    /**
     * Spektr, ktor tvoria chromozm, bud vystaven mutcii.
     * @param mutPower Sila mutcie.
     */
    public void mutate(double mutPower) {
        for(int i = 0; i < m; i++)
            spectrum[i].mutate(mutPower);
    }
    
    /**
     * Obnov hodnoty v spektrlnej matici.
     */
    public void updateA() {
        double[] a;
        int i, j;
        for(j = 0; j < m; j++) {
            a = spectrum[j].computeSpectrum();
            for(i = 0; i < n; i++)
                A[i][j] = a[i];
        }
    }
    
    /**
     * Vygeneruje nov chromozm s rovankmi typmi spektier.
     */
    public void newStart() {
        for(int i = 0; i < m; i++)
            spectrum[i].newStart();
    }
    
    /**
     * Vytvor kpiu objektu. <CODE>clone()</CODE> bude zavolan aj pre vetky obejkty, ktor tto trieda obsahuje.
     * @return Vrti kpiu objektu.
     */
    public Object clone() {
        int i;
        Chromosome ch = null;
        try {
            ch = (Chromosome) super.clone();
        } catch (CloneNotSupportedException e) {
            e.printStackTrace();
        }
        ch.spectrum = new Spectrum[m];
        for(i = 0; i < m; i++)
            ch.spectrum[i] = (Spectrum)(this.spectrum[i].clone());
        ch.A = new double[n][m];
        for(i = 0; i < n; i++)
            ch.A[i] = this.A[i].clone();
        return ch;
    }
}
