/*
 * FastICA.java
 *
 * Created on Pondelok, 2007, aprl 30, 20:31
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package spectKonfo;

import Jama.*;
/**
 * V tejto triede je implementovan algoritmus FastICA
 * @author Kamil Paulny
 */
public class FastICA {
    private double[][] y; //pole vstupov
    private int out; //dimenzia vystupu
    private int in; //dimenzia vstupu
    private String[] fileList; //cesty k vstupnm obrzkom
    private double[][] w; //vhov matica
    private int width, height, pixelCount; //rka a vska obrzka, poet pixelov
    private double accuracy; //presnos poda ktorej sa ziuje, i matica skonvergovala.
    /**
     * Predvolen presnos poda ktorej sa ziuje, i matica skonvergovala.
     */
    public static final double ACCURACY_DEFAULT=0.001;
    
    /**
     * 
     * @param outputDim Dimenzia vstupu
     * @param fileList Cesty k obrzkom
     * @param accuracy presnos poda ktorej sa ziuje, i matica skonvergovala.
     */
    
    
    public FastICA(int outputDim, String[] fileList, double accuracy) {
        in = fileList.length;
        out = outputDim;
        this.fileList = fileList;
        this.accuracy = accuracy;
        y = IO.loadImagesDouble(fileList);
        width = IO.getWidth(fileList[0]);
        height = IO.getHeight(fileList[0]);
        pixelCount = width * height;
    }
    
    /**
     * 
     * @param outputDim Dimenzia vstupu
     * @param fileList Cesty k obrzkom
     */
    public FastICA(int outputDim, String[] fileList) {
        this(outputDim, fileList, ACCURACY_DEFAULT);
    }
    
    private void prepare() {
        w = new double[out][out];
        for(int i = 0; i < out; i++)
            for(int j = 0; j < out; j++)
                w[i][j] = RandomUtils.nextDouble() * 0.05;
        if (in != out) {
            PCA pca = new PCA(out, y);
            y = pca.preprocessingICA();
        }
        Preprocessing prep = new Preprocessing();
        prep.fastICA(y);
        
    }
    
    /**
     * Spust algoritmus. Vsledky sa uloia v pracovnom adresri.
     */
    public void run() {
        prepare();
        train();
        saveOutput();
        
    }
    
    //Toto je napisane velmi necitatelne, uz sa ani sam neodvazim to opravovat alebo popisat.
    private void train() {
        int i, j, k;
        Matrix W, newW = new Matrix(w), oldW = new Matrix(out, out);
        double norm = 1000;
        double[] alfa = new double[out], beta = new double[out];
        double[][] cov = new double[out][out]; //E{g(y)yT}
        double[][] x = new double[pixelCount][out]; //vystup
        
        Matrix diagA, diagB, Cov;
        
        while(norm > (accuracy * out)) {
            //symetricka dekorelacia w
            W = newW;
            Matrix wwt = W.times(W.transpose());
            norm = wwt.norm2();
            norm = Math.sqrt(norm);
            W = W.times(1 / norm);
            newW = (W.times(3)).minus(W.times((W.transpose()).times(W))).times(0.5);
            while((W.minus(newW)).norm2() > (accuracy * out)) {
                W = newW;
                newW = (W.times(3)).minus(W.times((W.transpose()).times(W))).times(0.5);
            }
            norm = (oldW.minus(newW)).norm2();
            System.out.println(norm);
            w = newW.getArray();
            
            //vypocitanie vystupu
            for(i = 0; i < pixelCount; i++)   {
                for(j = 0; j < out; j++)   {
                    x[i][j] = 0;
                    for(k = 0; k < out; k++)
                        x[i][j] += w[j][k] * y[i][k];
                }
            }
            //vypocitanie cov
            for(j = 0; j < out; j++)
                for(k = 0; k < out; k++)
                    cov[j][k] = 0;
            for(i = 0; i < pixelCount; i++)
                for(j = 0; j < out; j++)
                    for(k = 0; k < out; k++)
                        cov[j][k] += g(x[i][j]) * x[i][k];
            for(j = 0; j < out; j++)
                for(k = 0; k < out; k++)
                    cov[j][k] /= pixelCount;
            
            //vypocitanie beta
            for(j = 0; j < out; j++)
                beta[j] = 0;
            for(i = 0; i < pixelCount; i++)
                for(j = 0; j < out; j++)
                    beta[j] += x[i][j] * g(x[i][j]);
            for(j = 0; j < out; j++)
                beta[j] /= -pixelCount;
            
            //vypocitanie alfa
            for(j = 0; j < out; j++)
                alfa[j] = 0;
            for(i = 0; i < pixelCount; i++)
                for(j = 0; j < out; j++)
                    alfa[j] += dg(x[i][j]);
            for(j = 0; j < out; j++) {
                alfa[j] /= pixelCount;
                alfa[j] = -1 / (beta[j] - alfa[j]);
            }
            
            //vypocitanie novej matice w
            Cov = new Matrix(cov);
            diagA = new Matrix(out,out);
            for(j = 0; j < out; j++)
                diagA.set(j, j, alfa[j]);
            diagB = new Matrix(out, out);
            for(j = 0;j < out; j++)
                diagB.set(j, j, beta[j]);
            W = new Matrix(w);
            newW = diagB.plus(Cov);
            newW = diagA.times(newW.times(W));
            newW = W.plus(newW);
            oldW = W;
            
        }
        
    }
    
    private void saveOutput() {
        int i, j, k;
        double[][] x = new double[pixelCount][out];
        Matrix W = new Matrix(w); // Matrix W=Matrix.identity(out,out);//ak chcem len vysledok z prep
        for(i = 0; i < pixelCount; i++)   {
            for(j = 0; j < out; j++)   {
                x[i][j]=0;
                for(k = 0; k < out; k++)
                    x[i][j] += w[j][k] * y[i][k];
            }
        }
        
        double max[] = new double[out];
        double min[] = new double[out];
        
        for(i = 0; i < pixelCount; i++)
            for(j = 0; j < out; j++) {
            // if(x[i][j] < 0) x[i][j] = 0;
            if(x[i][j] > max[j]) max[j] = x[i][j];
            if(x[i][j] < min[j]) min[j] = x[i][j];
            }
        for(i = 0; i < pixelCount; i++)
            for(j = 0; j < out; j++)
                x[i][j]=((x[i][j] - min[j]) / (max[j] - min[j])) * 255;
        
        for(i = 0; i < out; i++)
            IO.saveOutput("ica" , width , height , x , i);
        
        
    }
    
    private double g(double u) {
        return u*Math.exp(-0.5*Math.pow(u,2));
    }
    
    private double dg(double u) {
        return (1-Math.pow(u,2))*Math.exp(-0.5*Math.pow(u,2));
    }
}
