/*
 * GLMSpectrum.java
 *
 * Created on Nedea, 2007, aprl 29, 17:16
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package spectKonfo;

/**
 * Trieda <CODE>GLMSpectrum</CODE> zastupuje spektr, ktor maj tvar stu viacerch funkci hustoty Gaussovho a/alebo Lorentzovho rozdelenia.
 * @author Kamil Paulny
 */
public class GLMSpectrum extends Spectrum{
    /**
     * Predvolen poet nhodnch rozdelen, ktor tvoria toto spektrum.
     */
    public static final int GLCOUNT_DEFAULT = 2;
    private final double PCROSS = 0.5; //pravdepodobnos krenia
    private GLSpectrum[] gls; //GLMSpectrum je tvoren stom tchto spektier
    private int glCount;  //vekos poa gls
    /**
     * Vytvor nov intanciu triedy <CODE>GLMSpectrum</CODE>. Premennm <CODE>minWaveLength</CODE> a <CODE>step</CODE> prirad predvolen hodnoty. Spektrum bude tvoren predvolenm potom nhodnch rozdelen.
     * @param n Poet pouitch vlnovch dok
     */
    public GLMSpectrum(int n) {
        this(n, MIN_WAVE_LENGTH_DEFAULT, STEP_DEFAULT, GLCOUNT_DEFAULT);
    }
    
    
    /**
     * Vytvor nov intanciu triedy <CODE>GLMSpectrum</CODE>. Spektrum bude tvoren predvolenm potom nhodnch rozdelen.
     * @param n Poet pouitch vlnovch dok.
     * @param minWaveLength Doln hranicu rozsahu vlnovch dok pouitho svetla.
     * @param step Odstup medzi vlnovmi dkami.
     */
    public GLMSpectrum(int n, double minWaveLength, double step) {
        this(n, minWaveLength, step, GLCOUNT_DEFAULT);
        
    }
    
    /**
     * Vytvor nov intanciu triedy <CODE>GLMSpectrum</CODE>.
     * @param n Poet pouitch vlnovch dok.
     * @param minWaveLength Doln hranicu rozsahu vlnovch dok pouitho svetla.
     * @param step Odstup medzi vlnovmi dkami.
     * @param glCount Poet nhodnch rozdelen, ktor sa podieaj na tomto spektre.
     */
    public GLMSpectrum(int n, double minWaveLength, double step, int glCount) {
        super(n, minWaveLength, step);
        this.glCount = glCount;
        gls = new GLSpectrum[glCount];
        for(int i = 0; i < glCount; i++)
            gls[i] = new GLSpectrum(n, minWaveLength, step);
    }
    
    /**
     * Vrti hodnoty spektra v bodoch <CODE>minWavelength + i * step</CODE> kde <CODE>0 <= i < n </CODE>. Hodnoty spektra s nezporn.
     * @return Vrti pole hodnt spektra.
     */
    public double[] computeSpectrum() {
        double[] a1 = new double[n], a2 = new double[n];
        int i, j;
        for(i = 0; i < glCount; i++) {
            a2 = gls[i].computeSpectrum();
            for(j = 0; j < n; j++)
                a1[j] += a2[j];
        }
        return a1;
    }
    
    /**
     * Parametre spektra bud vystaven mutcii.
     * @param mutPower Sila mutcie. Hodnota by mala by z intervalu < 0 , 1 >.
     */
    public void mutate(double mutPower) {
        int i;
        for(i = 0; i < glCount; i++)
            gls[i].mutate(mutPower);
    }
    
    
    /**
     * Prebehne krenie medzi tmto spektrom a spektrom <CODE>s</CODE>.
     * @param s pektrum s ktorm prebehne krenie.
     */
    public void cross(Spectrum s) {
        GLSpectrum gl;
        for(int i = 0; i < glCount; i++) {
            if (RandomUtils.nextDouble() < PCROSS) {
                gl = this.gls[i];
                this.gls[i] = ((GLMSpectrum)s).gls[i];
                ((GLMSpectrum)s).gls[i] = gl;
            }
            this.gls[i].cross(((GLMSpectrum)s).gls[i]);
        }
    }
    
    
    /**
     * Vygeneruje nov spektrum
     */
    public void newStart() {
        for(int i = 0; i < glCount; i++)
            gls[i] = new GLSpectrum(n, minWaveLength, step);
    }
    
    /**
     * Vytvor kpiu obejktu. <CODE>clone()</CODE> bude zavolan aj pre vetky objekty, ktor tto trieda obsahuje.
     * @return Vrti kpiu objektu.
     */
    public Object clone() {
        GLMSpectrum gl = null;
        gl = (GLMSpectrum) super.clone();
        gl.gls=new GLSpectrum[glCount];
        for(int i = 0; i < glCount; i++)
            gl.gls[i] = (GLSpectrum)this.gls[i].clone();
        
        return gl;
    }
}
