/*
 * GLSpectrum.java
 *
 * Created on Nedea, 2007, aprl 29, 14:06
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package spectKonfo;

/**
 * Trieda <CODE>GLSpectrum</CODE> zastupuje spektr, ktor maj tvar funkcie hustoty Gaussovho alebo Lorentzovho rozdelenia.
 * @author Kamil Paulny
 */
public class GLSpectrum extends Spectrum {
    
    private double ni;       //Stredn hodnota
    private double sigma;    //Odmocnina z disperzie
    private double multiple; //Nsobok rozdelenia.
    
    private final int GAUSS = 1;
    private final int LORENTZ = 0;
    private final int TYPE_COUNT = 2; //Poet rznych typov spektier - po zmenen tejto hodnoty je nutn zmeni mutate()
    private int type; //type oznauje typ spektra. Nadobda hodnoty vyie uvedench kontnt
    
    private final double NI_RANGE = 2;      //ni sa moze pohybovat medzi -NI_RANGE az n-1+NI_RANGE
    private final double MIN_SIGMA_GEN = 5; //Minimlna hodnota sigmy pri generovan spektra
    private final double MAX_SIGMA_GEN = 15;//Maximlna hodnota sigmy pri generovan spektra
    private final double P_MUT = 0.5;       //Pravdepodobnos mutcie
    private final double P_MUT_NEW = 0.015; //Pravdepodobnos prepsania prvku novm
    private final double P_TYPE_MUT = 0.05; //Pravdepodobnost zmutovania typu
    private double sigmaMultiple = 1E12; //slo, ktorm treba prensobi sigma pred pouitm vo vzorci. Tto hodnota je vhodn pre rozsah 500-700nm
    
    
    /**
     * Vytvor nov intanciu triedy <CODE>GLSpectrum</CODE>. Premennm <CODE>minWaveLength</CODE> a <CODE>step</CODE> prirad predvolen hodnoty.
     * @param n Poet pouitch vlnovch dok.
     */
    public GLSpectrum(int n) {
        this(n, MIN_WAVE_LENGTH_DEFAULT, STEP_DEFAULT);
    }
    
    /**
     * Vytvor nov intanciu triedy <CODE>GLSpectrum</CODE>.
     * @param n Poet pouitch vlnovch dok.
     * @param minWaveLength Doln hranicu rozsahu vlnovch dok pouitho svetla.
     * @param step Odstup medzi vlnovmi dkami.
     */
    public GLSpectrum(int n, double minWaveLength, double step) {
        super(n, minWaveLength, step);
        ni = RandomUtils.nextDoubleRange(-NI_RANGE, n - 1 + NI_RANGE);
        sigma = RandomUtils.nextDoubleRange(MIN_SIGMA_GEN, MAX_SIGMA_GEN);
        multiple = RandomUtils.nextDoubleRange(0, 1);
        type=RandomUtils.nextInt(TYPE_COUNT);
        
    }
    
    /**
     * Vrti hodnoty spektra v bodoch <CODE>minWavelength + i * step</CODE> kde <CODE>0 <= i < n </CODE>. Hodnoty spektra s nezporn.
     * @return Vrti pole hodnt spektra.
     */
    public double[] computeSpectrum() {
        double[] a = new double[n];
        for(int i = 0; i < n; i++)
            a[i] = value(i);
        return a;
    }
    
    /**
     * Parametre spektra bud vystaven mutcii.
     * @param mutPower Sila mutcie. Hodnota by mala by z intervalu < 0 , 1 >.
     */
    public void mutate(double mutPower) {
        
        if (RandomUtils.nextDouble() < P_MUT)
            ni += RandomUtils.nextGaussian() * mutPower;
        if (RandomUtils.nextDouble() < P_MUT)
            sigma += RandomUtils.nextGaussian() * mutPower;
        if (RandomUtils.nextDouble() < P_MUT)
            multiple += RandomUtils.nextGaussian() * mutPower / 20.0;
        
        if (RandomUtils.nextDouble() < P_MUT_NEW)
            ni = RandomUtils.nextDoubleRange(-NI_RANGE, n + NI_RANGE);
        if (RandomUtils.nextDouble() < P_MUT_NEW)
            sigma = RandomUtils.nextDoubleRange(MIN_SIGMA_GEN, MAX_SIGMA_GEN);
        if (RandomUtils.nextDouble() < P_MUT_NEW)
            multiple = RandomUtils.nextDoubleRange(0, 1);
        if (RandomUtils.nextDouble() < P_TYPE_MUT)
            type = 1 - type;
        
        if (ni < -NI_RANGE)
            ni = -NI_RANGE;
        if (ni > n + NI_RANGE)
            ni = n + NI_RANGE;
        if (sigma <= 0)
            sigma = 1E-10;
        if (multiple < 0)
            multiple = 0.01;
    }
    
    
    /**
     * Prebehne krenie medzi tmto spektrom a spektrom <CODE>s</CODE>.
     * @param s Spektrum s ktorm prebehne krenie.
     */
    public void cross(Spectrum s) {
        ;
    }
    
    /**
     * Vygeneruje nov spektrum
     */
    public void newStart() {
        ni = RandomUtils.nextDoubleRange(-NI_RANGE, n - 1 + NI_RANGE);
        sigma = RandomUtils.nextDoubleRange(MIN_SIGMA_GEN, MAX_SIGMA_GEN);
        multiple = RandomUtils.nextDoubleRange(0, 1);
        type=RandomUtils.nextInt(TYPE_COUNT);
        
    }
    
    //Vzorce na hustoty rozdelen
    private double value(int x) {
        double r=3E14 * multiple;
        if (type == 0) return r*(1/Math.PI)*(valueSigma(sigma)/(Math.pow(valueX(x)-valueX(ni),2)+Math.pow(valueSigma(sigma),2)));
        else return r*(1/(valueSigma(sigma)*Math.sqrt(2*Math.PI)))*Math.exp(-0.5*Math.pow((valueX(x)-valueX(ni))/valueSigma(sigma),2));
        
    }
    
    //Hustotu nepotame na vlnovch dkach ale na frekvencich svetla
    private double valueX(double x) {
        return 3E17 / (x * step + minWaveLength);
    }
    
    
    private double valueSigma(double sigma) {
        return sigma * sigmaMultiple;
    }
    
    /**
     * Vytvor kpiu obejktu. <CODE>clone()</CODE> bude zavolan aj pre vetky objekty, ktor tto trieda obsahuje.
     * @return Vrti kpiu objektu.
     */
    public Object clone() {
        GLSpectrum gl = null;
        gl = (GLSpectrum) super.clone();
        
        return gl;
    }
    
    
}
