/*
 * GenAlg.java
 *
 * Created on Nedea, 2007, aprl 29, 19:21
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package spectKonfo;

import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

/**
 * V tejto triede je implementovan genetick algoritmus, ktor sa sna minimalizuje funkcie f1 a f4 popsan v diplomovej prci.
 * @author Kamil Paulny
 */
public class GenAlg {
    /**
     * Predvolen hodnota pre poet chromozmov
     */
    public static final int CHROM_COUNT_DEFAULT = 100;
    /**
     * Predvolen hodnota pre poet pouvanch vstupov
     */
    public static final int INPUT_COUNT_DEFAULT = 100;
    /**
     * Predvolen hodnota pre poet epoch
     */
    public static final int EPOCH_COUNT_DEFAULT = 50;
    /**
     * Kontanta pre funkciu f1.
     */
    public static final int ERROR_SUM = 1;
    /**
     * Kontanta pre funkciu f4
     */
    public static final int ERROR_SUM_KURT = 4;
    /**
     * Predvolen typ funkcie.
     */
    public static final int FUNCTION_TYPE_DEFAULT = ERROR_SUM;
    private int chromCount; //poet chromozmov
    private int inputCount; //poet vstupov vybratch kad epochu
    private int allInputCount; //poet vetkch vstupov
    private int epochCount; //poet epoch
    private int functionType; //typ funkcie. Mon hodnoty s Error_SUM a ERROR_SUM_KURT
    private char[][] y; //vybran vstupy
    private char[][] input; //vetky vstupy
    private Chromosome[] chr; //chromozmy
    private Chromosome[] chrNew; //nov populcia chromozmov
    private String[] fileList; //cesty k vstupnm obrzkom
    private int n, m; //rozmery spektrlnych matc
    private ExecutorService es;
    
    
    /**
     * 
     * @param m Dimenzia vstupu
     * @param s Cesty k obrzkom
     * @param type Typy spektier. Ak dka tohto poa nie je zhodn s m, tak s pouit predvolen hodnoty.
     */
    
    public GenAlg(int m, String[] s, int[] type) {
        this(m, s, type, Spectrum.MIN_WAVE_LENGTH_DEFAULT, Spectrum.STEP_DEFAULT, FUNCTION_TYPE_DEFAULT, CHROM_COUNT_DEFAULT, INPUT_COUNT_DEFAULT, EPOCH_COUNT_DEFAULT);
    }
    
    /**
     * 
     * @param m Dimenzia vstupu
     * @param s Cesty k obrzkom
     * @param type Typy spektier. Ak dka tohto poa nie je zhodn s m, tak s pouit predvolen hodnoty.
     * @param minWaveLength Najniia pouit vlnov dka.
     * @param step Rozdiel medzi vlnovmi dkami.
     * @param functionType Typ minimalizovanej funkcie
     */
    public GenAlg(int m, String[] s, int[] type, double minWaveLength, double step, int functionType) {
        this(m, s, type, minWaveLength, step, functionType, CHROM_COUNT_DEFAULT, INPUT_COUNT_DEFAULT, EPOCH_COUNT_DEFAULT);
    }
        
    /**
     * 
     * @param m Dimenzia vstupu
     * @param s Cesty k obrzkom
     * @param type Typy spektier. Ak dka tohto poa nie je zhodn s m, tak s pouit predvolen hodnoty.
     * @param minWaveLength Najniia pouit vlnov dka.
     * @param step Rozdiel medzi vlnovmi dkami.
     * @param functionType Typ minimalizovanej funkcie
     * @param chromCount poet chromozmov
     * @param inputCount poet vstupov pouvanch na vpoet funkcie
     * @param epochCount poet epoch
     */
    public GenAlg(int m, String[] s, int[] type, double minWaveLength, double step, int functionType, int chromCount, int inputCount, int epochCount) {
        int i, j;
        this.n = s.length;
        this.m = m;
        this.chromCount = chromCount;
        this.inputCount = inputCount;
        this.epochCount = epochCount;
        this.functionType = functionType;
        if ((type == null) || (type.length != m)) {
            this.m++;
            type = new int[this.m];
            for (i = 0; i < this.m-1; i++)
                type[i] = Chromosome.TYPE_DEFAULT;
            type[this.m-1] = Chromosome.UNIFORM;
        }
        allInputCount = IO.getPixelCount(s[0]);
        input = IO.loadImages(s);
        fileList = s;
        chr = new Chromosome[chromCount];
        chrNew = new Chromosome[chromCount];
        for (i = 0; i < chromCount; i++)
            chr[i] = new Chromosome(this.n, type, minWaveLength, step);
        
    }
    
    
    
    /**
     * Spust algortimus raz z vrti vazn spektrlnu maticu.
     * @return Vrti spektrlnu maticu
     */
    public double[][] run() {
        es = Executors.newFixedThreadPool(16);
        evolution();
        int win = findWinner();
        double[][] winner = new double[n][m];
        for (int i = 0; i < n; i++)
            winner[i] = chr[win].A[i].clone();
        es.shutdown();
        normA(winner);
        return winner;
    }
    
    /**
     * Spust algoritmus k krt a vrti vazn spektrlnu maticu
     * @param k poet spusten algoritmu
     * @return Vrti spektrlnu maticu
     */
    public double[][] bestOf(int k) {
        double[][] winner = null, A;
        double min = 2E20, d ;
        for (int i = 0; i < k; i++) {
            newStart();
            A = run();
            LSE lse=new LSE(input, A);
            if (functionType == ERROR_SUM) d = lse.computeErrorSum();
            else d = lse.computeKurtAndErrorSum();
        //    System.out.println(d);
            if (d < min) {
                min = d;
                winner = A;
            }
            
        }
        
        return winner;
        
    }
    
    //Vygeneruje nov chromozmy s rovankmi charakteristikami
    private void newStart() {
        for(int i = 0; i < chromCount; i++)
            chr[i].newStart();
    }
    
    /**
     * Ulo vsledne obrzky do pracovnho adresra.
     * @param winner spektrlna matica
     */
    public void saveOutput(double[][] winner) {
        LSE lse=new LSE(input, winner);
        double[][] x = lse.computeOutput();
        int width = IO.getWidth(fileList[0]), height = allInputCount / width;
        for(int i = 0; i < m; i++)
            IO.saveOutput("Genalg", width , height , x , i);
    }
    
    //uprav maticu tak, aby maximum v kadom stpci bolo 1
    private void normA(double A[][]) {
        int i, j;
        double max;
        for(j = 0; j < A[0].length; j++) {
            max = 0;
            for(i = 0; i < A.length;i++)
                if (A[i][j] > max)
                    max = A[i][j];
            for(i = 0; i < A.length;i++)
                A[i][j] /= max;
        }
    }
    private int findWinner() {
        int i;
        inputCount *= 5;
        updateA();
        chooseInput();
        fitness();
        double max = 0;
        int winner = 0;
        for (i = chromCount-1; i > 0; i--)
            if (chr[i].fitness - chr[i-1].fitness > max) {
            max = chr[i].fitness - chr[i-1].fitness;
            winner = i;
            }
        if (chr[0].fitness > max)
            winner = 0;
        inputCount /= 5;
        return winner;
    }
    
    private void evolution() {
        int i;
        for (i = 0; i < epochCount; i++) {
            updateA();
            chooseInput();
            fitness();
            newPopulation();
            cross();
            mutate(i);
        }
        
    }
    
    private void mutate(int epochNumber) {
        double mutPower = 1-(1.0*epochNumber/epochCount);
        for(int i = 0; i < chromCount; i++)
            chr[i].mutate(mutPower);
    }
    
    private void cross() {
        for(int i = 1; i < chromCount; i = i + 2)
            chr[i].cross(chr[i - 1]);
    }
    
    private void newPopulation() {
        int i, j;
        double d;
        for (i = 0; i < chromCount; i++) {
            d = RandomUtils.nextDouble();
            j = 0;
            while ((chr[j].fitness < d) && (j < chromCount-1))
                j++;
            chrNew[i] = (Chromosome)chr[j].clone();
        }
        for (i = 0; i < chromCount; i++) {
            chr[i] = chrNew[i];
        }
    }
    
    private class Task implements Callable<Double> {
        private double[][] A;
        
        public Task(double[][] A) {
            this.A = A;
        }
        
        public Double call() {
            LSE lse=new LSE(y, A);
            if (functionType == ERROR_SUM_KURT)
                return  lse.computeKurtAndErrorSum()+1E-10;
            else
                return lse.computeErrorSum()+1E-10;
        }
    }
    
    
    
    private void fitness() {
        int i;
        double minFit = 2e20, maxFit = -2e20;
        LSE lse;
        double sum = 0;
        Future<Double>[] future = new Future[chromCount];
        
        for (i = 0; i < chromCount; i++) {
            Task task = new Task(chr[i].A);
            future[i] = es.submit(task);
        }
        try {
            for (i = 0; i < chromCount; i++)  {
                chr[i].fitness = future[i].get();
                if (chr[i].fitness < minFit)
                    minFit = chr[i].fitness;
                if (chr[i].fitness > maxFit)
                    maxFit = chr[i].fitness;
            }
        } catch(Exception ie) {
            ie.printStackTrace();
        }
        
        // System.out.println(minFit);
        // System.out.println(maxFit);
        maxFit = minFit + 0.1*minFit + (maxFit-minFit)/60;
        for (i = 0; i < chromCount; i++) {
            if (chr[i].fitness > maxFit)
                chr[i].fitness = maxFit;
            chr[i].fitness = (0.99 * chr[i].fitness + 0.01*minFit - maxFit)/(minFit - maxFit);
            sum += chr[i].fitness;
        }
        for (i = 0; i < chromCount; i++) 
            chr[i].fitness /= sum;
        for (i = 1; i < chromCount; i++)
            chr[i].fitness += chr[i-1].fitness;
        
    }
    
    private void chooseInput() {
        int i, j;
        y = new char[inputCount][n];
        for (i = 0; i < inputCount; i++) {
            j = RandomUtils.nextInt(input.length);
            y[i] = input[j].clone();
        }
        
    }
    
    private void updateA() {
        for(int i = 0; i < chromCount; i++)
            chr[i].updateA();
    }
}
