/*
 * LSE.java
 *
 * Created on Nedea, 2007, aprl 29, 19:58
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package spectKonfo;

import Jama.*;

/**
 * Tto trieda sli na vpoet metdy LSE a potrebnch funkci s ou spojench.
 * @author Kamil Paulny
 */


public class LSE {
    private int n; //dimenzia vstupu
    private int m; //dimenzia vystupu
    private int inputCount; //pocet vstupov
    private char y[][]; //vstupy
    
    private Matrix A,B; //
    
    /**
     * Vytvor nov intanciu triedy LSE.
     * @param input Vstupy. Prv index oznauje index vektora. Druh index je slo pixelu.
     * @param C spektrlna matica
     */
    
    
    public LSE(char[][] input, double[][] C) {
        y = input;
        inputCount = input.length;
        A = new Matrix(C);
        n = A.getRowDimension();
        m = A.getColumnDimension();
        try {
            B = ((A.transpose().times(A)).inverse()).times(A.transpose());
        } catch(java.lang.RuntimeException ex) {
            B = new Matrix(m,n,0);
        }
    }
    
    
    /**
     * Vrti sumu Euklidovskch noriem chybovch vektorov. (funkcia f1 z dipl. prce)
     * @return
     */
    public double computeErrorSum() {
        int i, j, k;
        double errorSum = 0, e;
        double e0[] = new double[n];
        double[] y0, x0;
        double[][] b = B.getArray();
        double[][] a = A.getArray();
        
        for (i = 0; i < inputCount; i++) {
            x0 = new double[m];
            for (j = 0; j < m; j++) {
                for (k = 0; k < n; k++)
                    x0[j] += y[i][k]*b[j][k];
               /* if (x0[j] < 0)
                    x0[j] = 0;*/
            }
            y0 = new double[n];
            for (j = 0; j < n; j++)
                for (k = 0; k < m; k++)
                    y0[j] += x0[k]*a[j][k];
            
            for (j = 0; j < n; j++) {
                e0[j] = y[i][j] - y0[j];
                e0[j] *= e0[j];
            }
            e = 0;
            for(j = 0; j < n; j++)
                e += e0[j];
         /*   Arrays.sort(e0);
            e = e0[n-1];  //najvyssia chyba*/
            errorSum += e;
        }
        
        return errorSum;
    }
    
    /**
     * Vrti hodnota, ktor odvoden od hodnoty kurtosis a aj vsledku<CODE>computeErrorSum</CODE>. (funkcia f4 z diplomovej prce)
     * @return
     */
    public double computeKurtAndErrorSum() {
        int i, j, k;
        double errorSum = 0, e;
        double e0[] = new double[n];
        double[] y0, x0;
        double[][] b = B.getArray();
        double[][] a = A.getArray();
        double[] E = new double[m], E2 = new double[m], E4 = new double[m];
        double kurt = 0;
        for (i = 0; i < inputCount; i++) {
            x0 = new double[m];
            for (j = 0; j < m; j++) 
                for (k = 0; k < n; k++) 
                    x0[j] += y[i][k]*b[j][k];
                 
            for (j = 0; j < m; j++)
                E[j] += x0[j];
            y0 = new double[n];
            for (j = 0; j < n; j++)
                for (k = 0; k < m; k++)
                    y0[j] += x0[k]*a[j][k];
            
            for (j = 0; j < n; j++) {
                e0[j] = y[i][j] - y0[j];
                e0[j] *= e0[j];
            }
            e = 0;
            for(j = 0; j < n; j++)
                e += e0[j];
            errorSum += e;
        }
        
        for(j = 0; j < m; j++)
            E[j] /= inputCount;
        for (i = 0; i < inputCount; i++) {
            x0 = new double[m];
            for (j = 0; j < m; j++) 
                for (k = 0; k < n; k++)
                    x0[j] += y[i][k]*b[j][k];
              
            for(j = 0; j < m; j++) {
                E2[j] += Math.pow(x0[j] - E[j], 2);
                E4[j] += Math.pow(x0[j] - E[j], 4);
            }
        }
        
        for(j = 0; j < m; j++) {
            E2[j] /= inputCount;
            E2[j] *= E2[j];
            E4[j] /= inputCount;
            kurt += Math.abs((E4[j]/E2[j])-3);
        }
        
        errorSum = (1 / kurt) * errorSum;
        return errorSum;
    }
    
    /**
     * Vypota vstup metdy LSE.
     */
    public double[][] computeOutput() {
        int i, j, k;
        // System.out.println("Zacinam LSE");
        double maxPos[] = new double[m];
        double maxNeg[] = new double[m];
        double x[][] = new double[inputCount][m];
        double[] x0;
        double[][] b = B.getArray();
        
        for (i = 0; i < inputCount; i++) {
            x0 = new double[m];
            for (j = 0; j < m; j++)
                for (k = 0; k < n; k++)
                    x0[j] += y[i][k]*b[j][k];
            for (j = 0; j < m; j++) x[i][j] = x0[j];
            for (j = 0; j < m; j++) {
                if (x[i][j] > maxPos[j]) maxPos[j] = x[i][j];
                if (x[i][j] < 0) x[i][j] = 0;
                if (x[i][j] < maxNeg[j]) maxNeg[j] = x[i][j];
            }
        }
        for (j = 0; j < m; j++)
            maxPos[j] -= maxNeg[j];
        
        
        for (i = 0; i < inputCount; i++)
            for (j = 0; j < m; j++)
                x[i][j] = ((x[i][j] - maxNeg[j]) / maxPos[j]) * 255;
        
        return x;
    }
}
