/*
 * PCA.java
 *
 * Created on Pondelok, 2007, aprl 30, 18:17
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package spectKonfo;

/**
 * V tejto triede je implementovan PCA.
 * @author Kamil
 */
public class PCA {
    private int out; //dimenzia vystupu
    private int in; //dimenzia vstupu
    private double[][] y; //vstupy
    private double[][] W; //vahova matica
    private String[] fileList; //zoznam ciest k obrzkom
    private int width, height, pixelCount; //rozmery obrazkov, pocet pixelov (vstupov)
    
    /**
     * Kontruktor PCA, ak ho plnujeme poui ako samostatn metdu.
     */
    public PCA(int outputDim, String[] fileList) {
        out = outputDim + 1;
        this.fileList = fileList;
        y = IO.loadImagesDouble(fileList);
        in = y.length;
        width = IO.getWidth(fileList[0]);
        height = IO.getHeight(fileList[0]);
        pixelCount = width * height;
    }
  
    /**
     * Kontruktor pre PCA ak ho plnujeme poui ako predspracovanie pre FastICA.
     */
    public PCA(int outputDim, double[][] a) {
        y = a;
        in = a.length;
        out = outputDim + 1;
        fileList = null;
        pixelCount = y.length;
    }
    
    private double alfa(int t) {
        if (t < (pixelCount / 30)) return 0.1;
        else if ( t < (pixelCount / 5)) return 0.01;
        else return 0.001;
    }
    
    private void prepare() {
        W = new double[out][in];
        for(int i = 0; i < out ; i++)
            for(int j = 0; j < in; j++)
                W[i][j] = RandomUtils.nextDouble()-0.5;
        Preprocessing prep = new Preprocessing();
        prep.Pca(y);
    }
    
    
    /**
     * Spustenie PCA, ako samostatn metdu. Vsledky sa zap do pracovnho adresra.
     */
    public void run() {
        prepare();
        train();
        saveOutput();
    }
    
    
    /**
     * Spustenie pri pouit PCA ako predspracovania pre FastICA. Vrti upraven pole vstupov.
     */
    public double[][] preprocessingICA() {
        prepare();
        train();
        return computeOutput();
    }
    
    private void train() {
        int i,j,k,l,m;
        double[]  x;
        double[][] deltaW = new double[out][in];
        
        for(l = 0; l < pixelCount; l++) {
            m = RandomUtils.nextInt(pixelCount);
            x = new double[out];
            for (i = 0; i < out; i++)
                for (j = 0; j < in; j++)
                    x[i] += W[i][j]*y[m][j];
            for (j = 0; j < in; j++) {
                deltaW[0][j] = y[m][j] - x[0] * W[0][j];
                for(i = 1; i < out; i++)
                    deltaW[i][j] = deltaW[i-1][j] - x[i] * W[i][j];
                for(i = 0; i < out; i++)
                    deltaW[i][j] *= alfa(l)*x[i];
            }
            for (i = 0; i < out; i++)
                for (j = 0; j < in; j++)
                    W[i][j] += deltaW[i][j];
        }
        
    }
    
    private void saveOutput() {
        int i, j, k, l, m;
        double low[] = new double[out];
        double high[] = new double[out];
        double x[][] = computeOutput();
        for(i = 0; i < out; i++) {
            low[i] = 1E10;
            high[i] = -1E10;
        }
        for(m = 0; m < pixelCount; m++)
            for (i = 0; i < out; i++) {
            if (x[m][i] < low[i])  low[i]=x[m][i];
            if (x[m][i] > high[i])  high[i]=x[m][i];
            }
        for(i = 0; i < out; i++)
            high[i] -= low[i];
        
        for (m = 0; m < pixelCount; m++)
            for (i = 0; i < out; i++)
                x[m][i] = ((x[m][i] - low[i]) / high[i]) * 255;
        
        for(i = 0; i < out; i++)
            IO.saveOutput("pca" , width , height , x , i);
        
    }
    
    private double[][] computeOutput() {
        int i, j, m;
        out--;
        double x[][]=new double[pixelCount][out];
        for (m = 0; m < pixelCount; m++) {
            for (i = 0; i < out; i++)
                for (j = 0; j < in; j++)
                    x[m][i] += W[i][j]*y[m][j];
        }
        return x;
    }
    
}
