package spectKonfo;

/*
 * Spectrum.java
 *
 * Created on Nedea, 2007, aprl 29, 12:28
 *
 */

/**
 * Trieda <code>Spectrum</code> obsahuje premenn a metdy spolon pre vetky spektr. Spektrum udva silu vyarovania po osvetlen svetlom s danmi vlnovmi dlkami. Pouit vlnov dky musia ma medzi sebou rovnomern odstup, ktor je uveden v premennej <CODE>step</CODE>.
 * @author Kamil Paulny
 */
public abstract class Spectrum implements Cloneable{
    /**
     * Predvolen hodnota pre doln hranicu rozsahu vlnovch dok.
     */
    public static final double MIN_WAVE_LENGTH_DEFAULT = 500.0;
    /**
     * Predvolen hodnota odstupu medzi vlnovmi dkami.
     */
    public static final double STEP_DEFAULT = 10.0;
    /**
     * Doln hranicu rozsahu vlnovch dok pouitho svetla. Hodnota sa udva v nanometroch.
     */
    protected double minWaveLength; //Minimlna vlnov dka (v nanometroch) na ktorej spektrum sledujeme
    /**
     * Odstup medzi vlnovmi dkami.
     */
    protected double step; //Rozdiel medzi pouvanmi vlnovmi dkami
    /**
     * Poet pouitch vlnovch dok.
     */
    protected int n; //Poet rznych vlnovch dok (poet nasnmanch obrzkov)
    
    
    
    /**
     * Vytvor nov intanciu triedy <CODE>Spectrum</CODE>. Premennm <CODE>minWaveLength</CODE> a <CODE>step</CODE> prirad predvolen hodnoty.
     * @param n Poet pouitch vlnovch dok.
     */
    public Spectrum(int n) {
        this(n, MIN_WAVE_LENGTH_DEFAULT, STEP_DEFAULT);
    }
    
    /**
     * Vytvor nov intanciu triedy <CODE>Spectrum</CODE>.
     * @param n Poet pouitch vlnovch dok.
     * @param minWaveLength Doln hranicu rozsahu vlnovch dok pouitho svetla.
     * @param step Odstup medzi vlnovmi dkami.
     */
    public Spectrum(int n, double minWaveLength, double step) {
        this.n = n;
        this.minWaveLength = minWaveLength;
        this.step = step;
    }
    
    /**
     * Vrti hodnoty spektra v bodoch <CODE>minWavelength + i * step</CODE> kde <CODE>0 <= i < n </CODE>. Hodnoty spektra s nezporn.
     * @return Vrti pole hodnt spektra.
     */
    abstract public double[] computeSpectrum();
    /**
     * Parametre spektra bud vystaven mutcii.
     * @param mutPower Sila mutcie. Hodnota by mala by z intervalu < 0 , 1 >
     */
    abstract public void mutate(double mutPower);
    /**
     * Prebehne krenie medzi tmto spektrom a spektrom <CODE>s</CODE>
     * @param s Spektrum s ktorm prebehne krenie.
     */
    abstract public void cross(Spectrum s);
    /**
     * Vygeneruje nov spektrum
     */
    abstract public void newStart();
    
    /**
     * Vytvor kpiu obejktu. <CODE>clone()</CODE> bude zavolan aj pre vetky objekty, ktor tto trieda obsahuje.
     * @return Vrti kpiu objektu.
     */
    public Object clone() {
        Spectrum s = null;
        try {
            s = (Spectrum) super.clone();
        } catch (CloneNotSupportedException e) {
            e.printStackTrace();
        }
        return s;
    }
}
