package spectKonfo;
/*
 * UniversalSpectrum.java
 *
 * Created on Nedea, 2007, aprl 29, 13:27
 *
 */

/**
 * Trieda <CODE>UniversalSpectrum</CODE> zastupuje spektr, ktor nemaj iadne obmedzenia na svoj tvar.
 * @author Kamil Paulny
 */
public class UniversalSpectrum extends Spectrum {
    /**
     * Obsahuje hodnoty spektra v <CODE>n</CODE> bodoch. Vektor hodnt je udriavan v normovanom tvare. Hodnoty vektora s nezporn a vektor m jednotkov euklidovsku normu.
     */
    private double[] data;
    private final double PCROSS = 0.5;
    /**
     * Vytvor nov intanciu triedy <CODE>UniversalSpectrum</CODE>. Premenn <CODE>minWaveLength</CODE> a <CODE>step</CODE> nie s pre tento typ spektra dleit.
     * @param n Poet pouitch vlnovch dok
     */
    public UniversalSpectrum(int n) {
        super(n);
        data = RandomUtils.nextDoubleArray(n);
    }
    
    /**
     * Vrti hodnoty spektra v bodoch minWavelength + i * step kde 0 <= i < n. Hodnoty s nezporn a vektor hodnt je v Euklidovskom normalnom tvare.
     * @return Vrti pole hodnt spektra.
     */
    public double[] computeSpectrum() {
        return data.clone();
    }
    
    /**
     * Parametre spektra bud vystaven mutcii.
     * @param mutPower mutPower Sila mutcie. Hodnota by mala by z intervalu < 0 , 1 >
     */
    public void mutate(double mutPower) {
        int i;
        for(i = 0; i < n; i++) {
            data[i] += 0.03*mutPower * RandomUtils.nextGaussian();
            if (data[i] < 0)
                data[i] = 0;
        }
        normalize();
    }
    
    
    /**
     * Prebehne krenie medzi tmto spektrom a spektrom <CODE>s</CODE>.
     * @param s Spektrum s ktorm prebehne krenie.
     */
    public void cross(Spectrum s) {
        double d;
        for(int i = 0; i < n; i++)
            if (RandomUtils.nextDouble() < PCROSS) {
            d = this.data[i];
            this.data[i] = ((UniversalSpectrum)s).data[i];
            ((UniversalSpectrum)s).data[i] = d;
            }
        
    }
    
    /**
     * Vygeneruje nov spektrum
     */
    public void newStart() {
        data =  RandomUtils.nextDoubleArray(n);
    }
    
    //normalizuje vektor: jeho euklidovsk8 norma bude 1
    private void normalize() {
        int i;
        double sum = 0;
        for(i = 0; i < n; i++)
            sum += data[i]*data[i];
        sum = Math.sqrt(sum);
        for(i = 0; i < n; i++)
            data[i] /= sum;
    }
    
    /**
     * Vytvor kpiu objektu. <CODE>clone()</CODE> bude zavolan aj pre vetky obejkty, ktor tto trieda obsahuje.
     * @return Vrti kpiu objektu.
     */
    public Object clone() {
        UniversalSpectrum us = null;
        us = (UniversalSpectrum) super.clone();
        
        us.data = this.data.clone();
        
        return us;
    }
    
}
